<?php
/**
 * @file
 * Provides a helper class for working with links. This is designed to provide
 * functions for URL manipulations without requiring a LinkApiLink object.
 */
class LinkApiHelper {

  /**
   * Normalise a URL
   *
   * This takes a given URL and runs all the available tidying and sorting
   * functions over it.
   */
  static public function normaliseURL($url){

    // Remove session IDs
    $url = LinkApiHelper::stripSessionID($url);

    // Sort the query string, if necessary
    $url_parts = parse_url($url);
    if(isset($url_parts['query']) && !empty($url_parts['query'])) {
      $url_parts['query'] = LinkApiHelper::sortQueryString($url_parts['query'], LINK_API_QUERY_SORT_CLEAN);
    }

    // Build the URL
    $url = LinkApiHelper::buildURL($url_parts);

    // Add a trailing slash
    $url = LinkApiHelper::trailingSlash($url);

    return $url;
  }

  /**
   * If a link consists only of a protocol and hostname, add a trailing
   * slash.
   *
   * For example, "http://www.example.com" becomes "http://www.example.com/".
   *
   * This explicitly requests the default or index document, which slightly
   * speeds up page retrieval for the visitor, and normalises the URL in the
   * database
   *
   * @see Originally from the links module
   *
   * @param string $url
   *  A URL
   *
   * @return string
   *  The URL with a trailing slash
   */
  static public function trailingSlash($url) {
    $url = trim($url);
    if (preg_match('!^[A-Z0-9]+://[^/?]+$!i', $url)) {
      $url .= '/';
    }
    return $url;
  }

  /**
   * Orders the parameters in an HTTP GET query string.
   *
   * This helps to detect URLs that are the same except for the
   * parameter order.
   * For exammple:
   *    xyz=98&def=yes&abc=49
   * becomes:
   *    abc=49&def=yes&xyz=98
   *
   * Accepts either "&" or the W3C-preferred "&amp;" as a delimiter
   * between parameters.
   *
   * @param string $query
   *  The query string
   * @param string $mode [optional]
   *  The method to use for sorting. Either LINK_API_QUERY_SORT_CLEAN to remove
   *  empty parameters, or LINK_API_QUERY_SORT_RETAIN, to retain empty parameters.
   *
   * @return string
   *  A sorted query string.
   */
  static public function sortQueryString($query, $mode = LINK_API_QUERY_SORT_CLEAN) {

    // Convert all escaped ampersands prior to parse_str, or the output is
    // wrong
    $query = str_replace('&amp;', '&', $query);

    parse_str($query, $results);
    ksort($results);

    // Remove empty query strings
    if ($mode == LINK_API_QUERY_SORT_CLEAN) {
      foreach ($results as $key => $result) {
        if (empty($result)) {
          unset ($results[$key]);
        }
      }
    }

    // Return a built query string using & as the separator, for consistency
    return http_build_query($results, NULL, '&');
  }

  /**
   * This function removes known session ID strings from a URL.
   *
   * These are transient data from a particular browser on a particular day, and
   * definitely do not belong in a links database. The search is not
   * case sensitive. This also works on a bare query string rather than
   * a full URL, and can be used that way.
   *
   * @see Originally from the links module
   *
   * @param string $url
   *
   * @return string
   *  The URL minus a session ID
   */
  static public function stripSessionID($url) {
    $url = trim($url);
    $url = preg_replace('/(^|[&?]|&amp;)[0-9A-Z_]*SESS(ION|_ID|ID)*=[0-9A-Z_]*/i','\1',$url);
    return LinkApiHelper::cleanupParameters($url);
  }

  /**
   * The preg_replace() calls some functions may leave some cruft in the URL,
   * such as "&&", "&?", or "?" or "&" as the first or last character. This
   * function clears that problem up.
   *
   * @deprecated
   *
   * @see Originally from the links module
   *
   * @param string $url
   *
   * @return string
   *  The URL minus untidy parameters
   */
  static public function cleanupParameters($url) {
    $patterns = array('/(\&amp;|\&)(\&amp;|\&)/', '/^(\&amp;|\&)/', '/\?(\&amp;|\&)/', '/(\&amp;|\&|\?)$/', '/(\&amp;|\&)\?/');
    $replaces = array('\1', '', '?', '', '?');
    return preg_replace($patterns, $replaces, $url);
  }

  /**
   * Build a URL from parts provided from parse_url()
   *
   * @param array $parts
   *  The URL parts to join
   *
   * @return string
   *  A URL
   */
  static public function buildURL($parts) {
    return ((isset($parts['scheme'])) ? $parts['scheme'] . '://' : '')
      . ((isset($parts['user'])) ? $parts['user'] . ((isset($parts['pass'])) ? ':' . $parts['pass'] : '') . '@' : '')
      . ((isset($parts['host'])) ? $parts['host'] : '')
      . ((isset($parts['port'])) ? ':' . $parts['port'] : '')
      . ((isset($parts['path'])) ? $parts['path'] : '')
      . ((isset($parts['query'])) ? '?' . $parts['query'] : '')
      . ((isset($parts['fragment'])) ? '#' . $parts['fragment'] : '');
  }

  /**
   * Sanitize a URL
   *
   * @param string $url
   *  A URL
   *
   * @return mixed
   *  The sanitized version of the URL
   */
  static public function sanitizeURL($url) {
    return filter_var($url, FILTER_SANITIZE_URL);
  }

  /**
   * Validate a URL
   *
   * Note that the function will only find ASCII URLs to be valid;
   * internationalized domain names (containing non-ASCII characters) will fail.
   *
   * Its possible that in PHP prior to 5.2.13/5.3.2, this may return TRUE for
   * some invalid URLs.
   *
   * @param string $url
   *  A URL
   *
   * @return bool
   *  TRUE if the URL validates, and FALSE if it does not
   */
  static public function validateURL($url) {
    $_url = filter_var($url, FILTER_VALIDATE_URL);
    if ($_url == FALSE) {
      return FALSE;
    }
    else {
      return TRUE;
    }
  }

  /**
   * Validate an IP address
   */
  static public function validateIP($ip) {
    $_ip = filter_var($ip, FILTER_VALIDATE_IP);
    if ($_ip == FALSE) {
      return FALSE;
    }
    else {
      return TRUE;
    }
  }
}
