<?php
/**
 * @file
 * A link
 */
class LinkApiLink {

  /**
   * The identifier for the link
   */
  public $lid;

  /**
   * The Link API link type
   */
  public $link_type;

  /**
   * The node ID this link is stored against
   */
  public $nid;

  /**
   * The realm - an optional, module supplied property
   */
  public $realm;

  /**
   * The title for the link
   */
  public $title;

  /**
   * The URL scheme
   */
  public $scheme;

  /**
   * The URL host
   */
  public $host;

  /**
   * The URL port
   */
  public $port;

  /**
   * The URL user
   */
  public $user;

  /**
   * The URL password
   */
  public $pass;

  /**
   * The URL path
   */
  public $path;

  /**
   * The URL query string
   */
  public $query;

  /**
   * The URL fragment
   */
  public $fragment;

  /**
   * MD5 of the full URL
   */
  public $md5_full;

  /**
   * MD5 of the URL host, path, query and fragment
   */
  public $md5_partial;

  /**
   * MD5 of the URI reference (path + query + fragment)
   */
  public $md5_uri;

  /**
   * The full URL
   */
  public $url;

  /**
   * Constructor
   */
  public function __construct($config = array()){
    if (!empty($config)) {
      $this->make($config);
    }
  }

  /**
   * Save a LinkApiLink to the database
   *
   * @return \LinkApiLink
   */
  public function create() {

    // Make sure all relevant keys are set
    $this->presave();
    // Invoke hook_link_api_presave
    module_invoke_all('link_api_link_presave', $this);
    // Save
    $result = drupal_write_record(LINK_API_DB_NAME, $this);
    // Invoke hook_link_api_update
    module_invoke_all('link_api_link_update', $this);

    if($result != SAVED_NEW) {
      drupal_set_message('There was a problem creating the link for '. $this->url(), 'warning');
    }

    return $this;
  }

  /**
   * Update a LinkApiLink in the database
   *
   * @return \LinkApiLink
   */
  public function update() {

    $this->presave();
    module_invoke_all('link_api_link_presave', $this);
    $result = drupal_write_record(LINK_API_DB_NAME, $this, array('lid'));
    module_invoke_all('link_api_link_update', $this);

    if($result != SAVED_UPDATED) {
      drupal_set_message('There was a problem saving link lid: '. $this->lid, 'warning');
    }

    return $this;
  }

  /**
   * Load a LinkApiLink from the database
   *
   * @param string $lid
   *  The Link ID to read.
   *
   * @return bool | \LinkApiLink
   */
  public function read($lid) {
    $query = "SELECT * FROM {" . LINK_API_DB_NAME . "} WHERE `lid` = %d";
    $result = db_fetch_array(db_query($query, $lid));

    if (!empty($result)) {
      $this->make($result);
      module_invoke_all('link_api_link_view', $this);
      return $this;
    }

    return FALSE;
  }

  /**
   * Update a LinkApiLink in the database
   *
   * @return \LinkApiLink
   */
  public function upsert() {

    $this->match();

    if (isset($this->lid) && !empty($this->lid)) {
      $this->update();
    }
    else {
      $this->create();
    }

    return $this;
  }

  /**
   * Delete a LinkApiLink from the database
   *
   * @return \LinkApiLink
   */
  public function delete() {

    // Invoke hook_link_api_delete()
    module_invoke_all('link_api_link_delete', $this);

    // Delete
    db_query('DELETE FROM {' . LINK_API_DB_NAME . '} WHERE `lid` = %d', $this->lid);

    return $this;
  }

  /**
   * Create this object from database settings
   */
  public function make(array $config) {
    foreach ($config as $key => $setting) {
      if(property_exists($this, $key)) {
        $this->{$key} = $setting;
      }
    }
  }

  /**
   * Prepare for saving
   */
  public function presave() {
    $this->md5_full();
    $this->md5_partial();
    $this->md5_path();

    if (isset($this->nid)) {
      $this->link_type = LINK_API_TYPE_NODE;
    }
    else {
      $this->link_type = LINK_API_TYPE_SOLO;
    }

    if (!isset($this->realm)) {
      $this->realm = LINK_API_REALM_DEFAULT;
    }
  }

  /**
   * Load a URL into this object
   *
   * @param string $url
   *  A URL
   * @param bool $normalise [optional]
   *  Whether to normalise or not, defaults to true.
   *
   * @return bool
   *  TRUE, if the URL was successfully loaded
   */
  public function loadURL($url, $normalise = LINK_API_URL_NORMALISE) {

    // Sanitize the URL before loading it, in case its malicious
    $url = LinkApiHelper::sanitizeURL($url);
    if ($url == FALSE) {
      return FALSE;
    }

    // @todo This results in a double parse_url() being called
    // Normalise if required.
    if ($normalise == LINK_API_URL_NORMALISE) {
      $url = LinkApiHelper::normaliseURL($url);
    }

    $existing = $this->url();
    if (!empty($existing)){
      // We're replacing an existing URL
      module_invoke_all('link_api_link_replace', $this, $url);
    }

    // Do the replacement
    $this->url = $url;
    $parts = parse_url($url);
    $this->make($parts);

    return TRUE;
  }

  /**
   * Compute the URL hash for the full url
   */
  public function md5_full() {
    $url = $this->url(LINK_API_URL_BUILD_FULL);
    $this->md5_full = md5($url);
    return $this->md5_full;
  }

  /**
   * Compute the URL hash for the partial URL
   */
  public function md5_partial() {
    $url = $this->url(LINK_API_URL_BUILD_PARTIAL);
    $this->md5_partial = md5($url);
    return $this->md5_partial;
  }

  /**
   * Compute the URL hash for the path
   */
  public function md5_path() {
    $url = $this->url(LINK_API_URL_BUILD_PATH);
    $this->md5_uri = md5($url);
    $this->md5_uri;
  }

  /**
   * Associate a node with this link
   */
  public function associateNode($nid) {
    $this->nid = $nid;
  }

  /**
   * Set a title
   */
  public function title($title) {
    $this->title = trim(check_plain($title));
  }

  /**
   * Set or return the realm
   *
   * @param string $realm [optional]
   *  The new realm name
   *
   * @return string
   *  The current realm name (may be NULL)
   */
  public function realm($realm = NULL) {
    if (!is_null($realm)) {
      $this->realm = $realm;
    }
    return $this->realm;
  }

  /**
   * Check if this link already has a database record, and if it does, set the
   * $lid appropriately
   *
   * @param bool $strict
   *  Enforce strict checking for non-Node links. When FALSE, links not associated
   *  with nodes will return TRUE if the URL matches. This is generally set to
   *  TRUE, so that duplicate links are not merged inadvertantly.
   *
   * @return bool
   */
  public function match($strict = TRUE) {

    // If we have an lid already, exit now
    if (isset($this->lid)) {
      return $this->lid;
    }

    $md5 = $this->md5_full();
    if (isset($this->nid)) {
      $this->link_type = LINK_API_TYPE_NODE;
    }
    else {
      $this->link_type = LINK_API_TYPE_SOLO;
    }

    if ($this->link_type == LINK_API_TYPE_NODE) {
      $query = "SELECT * FROM {" . LINK_API_DB_NAME . "} WHERE `md5_full` = %s AND `nid` = %d AND `title` = %s";
      $result = db_query($query, $md5, $this->nid, $this->title);
    }
    elseif ($this->link_type == LINK_API_TYPE_SOLO && $strict == TRUE) {
      $query = "SELECT * FROM {" . LINK_API_DB_NAME . "} WHERE `md5_full` = %s AND `link_type` = %s AND `title` = %s";
      $result = db_query($query, $md5, LINK_API_TYPE_SOLO, $this->title);
    }
    elseif ($this->link_type == LINK_API_TYPE_SOLO) {
      $query = "SELECT * FROM {" . LINK_API_DB_NAME . "} WHERE `md5_full` = %s AND `link_type` = %s";
      $result = db_query($query, $md5, LINK_API_TYPE_SOLO);
    }

    if (isset($result)) {
      while($item = db_fetch_array($result)) {
        if ($item['title'] == $this->title) {
          $this->lid = $item['lid'];
          return $this->lid;
        }
      }
    }

    return FALSE;
  }

  /**
   * Make a URL with different configuration options, and sanitize it.
   *
   * @param string $method [optional]
   *  A constant defining the parts to use to build the link. Can be either
   *  LINK_API_URL_BUILD_FULL (default), LINK_API_URL_BUILD_PARTIAL, or LINK_API_URL_BUILD_PATH.
   * @param array $keys [optional]
   *  If specified, $method is ignored, and only the keys specified are used.
   * @param bool $validate
   *  Whether to validate the URL
   *
   * @return string
   *  A URL string
   */
  public function url($method = LINK_API_URL_BUILD_FULL, $keys = array(), $validate = FALSE) {
    $final_parts = array();

    if (empty($keys)) {
      switch ($method) {
        case LINK_API_URL_BUILD_PARTIAL:
          $keys = array('host', 'path', 'query', 'fragment');
          break;
        case LINK_API_URL_BUILD_PATH:
          $keys = array('path');
          break;
        case LINK_API_URL_BUILD_FULL:
          $keys = array('scheme', 'host', 'port', 'user', 'pass', 'path', 'query', 'fragment');
          break;
      }
    }


    // Pull out keys from $this for use in the build process
    if (!empty($keys)) {

      foreach ($keys as $key) {
        if(property_exists($this, $key) && !empty($this->{$key})) {
          $final_parts[$key] = $this->{$key};
        }
      }

      // Build the URL
      $url = LinkApiHelper::buildURL($final_parts);
      $url = LinkApiHelper::sanitizeURL($url);

      // If validation is requested, run it now
      if ($validate == TRUE) {
        $valid = LinkApiHelper::validateURL($url);
        if ($valid == FALSE) {
          return FALSE;
        }
      }

      return $url;
    }

    return FALSE;
  }

  /**
   * Return all link properties
   *
   * @return array
   *  An array of link properties
   */
  public function properties() {
    $link = (array) $this;
    $link['url'] = $this->url();
    return $link;
  }
}
